@@ -15,7 +15,8 @@ namespace Tomboy.Notebooks
 		static Gtk.TreeModelSort sortedNotebooks;
 		static Gtk.TreeModelFilter filteredNotebooks;
 		static Dictionary<string, Gtk.TreeIter> notebookMap;
-		static object locker = new object ();
+		static object locker = new object ();		
+		public static bool AddingNotebook = false;
 		#endregion // Fields
 		
 		public delegate void NotebookEventHandler (Note note, Notebook notebook);
@@ -123,7 +124,12 @@ namespace Tomboy.Notebooks
 				if (notebook != null)
 					return notebook;
 				
-				notebook = new Notebook (notebookName);
+				try {
+					AddingNotebook = true;
+					notebook = new Notebook (notebookName);
+				} finally {
+					AddingNotebook = false;
+				}
 				iter = notebooks.Append ();
 				notebooks.SetValue (iter, 0, notebook);
 				notebookMap [notebook.NormalizedName] = iter;
@@ -278,6 +284,26 @@ namespace Tomboy.Notebooks
 		/// </param>
 		/// <returns>If successful, returns the newly created notebook.</returns>
 		public static Notebook PromptCreateNewNotebook (Gtk.Window parent)
+		{
+			return PromptCreateNewNotebook (parent, null);
+		}
+		
+		/// <summary>
+		/// Prompt the user to create a new notebook and if successful, move
+		/// the notes specified in the notesToAdd list into the new notebook.
+		/// </summary>
+		/// <param name="parent">
+		/// A <see cref="Gtk.Window"/>
+		/// </param>
+		/// <param name="notesToAdd">
+		/// A <see cref="List`1"/> of notes that should be added to the new
+		/// notebook.
+		/// </param>
+		/// <returns>
+		/// The newly created <see cref="Notebook"/> if successful or null
+		/// if there was a problem.
+		/// </returns>
+		public static Notebook PromptCreateNewNotebook (Gtk.Window parent, List<Note> notesToAdd)
 		{
 			// Prompt the user for the name of a new notebook
 			Notebooks.CreateNotebookDialog dialog =
@@ -298,6 +324,13 @@ namespace Tomboy.Notebooks
 				Logger.Warn ("Could not create notebook: {0}", notebookName);
 			} else {
 				Logger.Debug ("Created the notebook: {0} ({1})", notebook.Name, notebook.NormalizedName);
+				
+				if (notesToAdd != null) {
+					// Move all the specified notesToAdd into the new notebook
+					foreach (Note note in notesToAdd) {
+						NotebookManager.MoveNoteToNotebook (note, notebook);
+					}
+				}
 			}
 			
 			return notebook;
@@ -359,7 +392,7 @@ namespace Tomboy.Notebooks
 			if (note == null)
 				return false;
 			
-			// TODO: In the future we may want to allow notes
+			// NOTE: In the future we may want to allow notes
 			// to exist in multiple notebooks.  For now, to
 			// alleviate the confusion, only allow a note to
 			// exist in one notebook at a time.
@@ -384,6 +417,19 @@ namespace Tomboy.Notebooks
 			
 			return true;
 		}
+		
+		public static void FireNoteAddedToNoteBook (Note note, Notebook notebook)
+		{
+			if (NoteAddedToNotebook != null)
+				NoteAddedToNotebook (note, notebook);
+		}
+		
+		public static void FireNoteRemovedFromNoteBook (Note note, Notebook notebook)
+		{
+			if (NoteRemovedFromNotebook != null)
+				NoteRemovedFromNotebook (note, notebook);
+		}
+		
 		#endregion // Public Methods
 		
 		#region Private Methods
@@ -429,17 +475,17 @@ namespace Tomboy.Notebooks
 			}
 		}
 
-        /// <summary>
-        /// Filter out SpecialNotebooks from the model
-        /// </summary>
-        static bool FilterNotebooks (Gtk.TreeModel model, Gtk.TreeIter iter)
-        {
-        	Notebook notebook = model.GetValue (iter, 0) as Notebook;
-        	if (notebook == null || notebook is SpecialNotebook)
-        		return false;
-        	
-        	return true;
-        }
+	        /// <summary>
+	        /// Filter out SpecialNotebooks from the model
+	        /// </summary>
+	        static bool FilterNotebooks (Gtk.TreeModel model, Gtk.TreeIter iter)
+	        {
+	        	Notebook notebook = model.GetValue (iter, 0) as Notebook;
+	        	if (notebook == null || notebook is SpecialNotebook)
+	        		return false;
+	        	
+	        	return true;
+	        }
 		#endregion // Private Methods
 	}
 }
