@@ -2,6 +2,9 @@ using System;
 using System.Collections.Generic;
 using Mono.Unix;
 using System.Runtime.InteropServices;
+#if !WIN32 && !MAC
+using GtkBeans;
+#endif
 
 namespace Tomboy
 {
@@ -31,7 +34,9 @@ namespace Tomboy
 		{
 			this.note = note;
 			Image = new Gtk.Image (note_icon);
-
+#if HAS_GTK_2_16
+			this.SetAlwaysShowImage (true);
+#endif
 			if (show_pin) {
 				Gtk.HBox box = new Gtk.HBox (false, 0);
 				Gtk.Widget child = Child;
@@ -57,10 +62,10 @@ namespace Tomboy
 		static string GetDisplayName (Note note)
 		{
 			string display_name = note.Title;
-			int max_length = 100;
+			int max_length = (int) Preferences.Get (Preferences.MENU_ITEM_MAX_LENGTH);
 
 			if (note.IsNew) {
-				string new_string = Catalog.GetString(" (new)");
+				string new_string = Catalog.GetString (" (new)");
 				max_length -= new_string.Length;
 				display_name = Ellipsify (display_name, max_length)
 					+ new_string;
@@ -136,6 +141,8 @@ namespace Tomboy
 	{
 		TomboyTray tray;
 		TomboyPrefsKeybinder keybinder;
+		Gtk.Menu context_menu;
+		Gtk.ImageMenuItem sync_menu_item;
 
 		public TomboyTrayIcon (NoteManager manager)
 		{
@@ -167,7 +174,7 @@ namespace Tomboy
 		protected override void OnPopupMenu (uint button, uint activate_time)
 		{
 			if (button == 3)
-				GuiUtils.PopupMenu (MakeRightClickMenu (),
+				GuiUtils.PopupMenu (GetRightClickMenu (),
 				                    null, 
 				                    new Gtk.MenuPositionFunc (GetTrayMenuPosition));
 				
@@ -175,14 +182,17 @@ namespace Tomboy
 		
 		public void ShowMenu (bool select_first_item)
 		{
-			TomboyTrayUtils.UpdateTomboyTrayMenu (tray, null);
-			if (select_first_item)
-				tray.TomboyTrayMenu.SelectFirst (false);
+			if (context_menu != null)
+				context_menu.Hide ();
 
-			
-				
-			GuiUtils.PopupMenu (tray.TomboyTrayMenu, null, 
-				new Gtk.MenuPositionFunc (GetTrayMenuPosition));
+			tray.NoteManager.GtkInvoke (() => {
+				TomboyTrayUtils.UpdateTomboyTrayMenu (tray, null);
+				if (select_first_item)
+					tray.TomboyTrayMenu.SelectFirst (false);
+
+				GuiUtils.PopupMenu (tray.TomboyTrayMenu, null,
+					new Gtk.MenuPositionFunc (GetTrayMenuPosition));
+			});
 		}
 		
 		public void GetTrayMenuPosition (Gtk.Menu menu,
@@ -218,40 +228,71 @@ namespace Tomboy
 				Logger.Error ("Exception in GetTrayMenuPosition: " + e.ToString ());
 			}
 		}
-		
-		Gtk.Menu MakeRightClickMenu ()
+
+		void Preferences_SettingChanged (object sender, EventArgs args)
 		{
-			Gtk.Menu menu = new Gtk.Menu ();
+			// Update items based on configuration.
+			UpdateMenuItems ();
+		}
+
+		void UpdateMenuItems ()
+		{
+			// Is synchronization configured and active?
+			string sync_addin_id = Preferences.Get (Preferences.SYNC_SELECTED_SERVICE_ADDIN)
+				as string;
+			sync_menu_item.Sensitive = !string.IsNullOrEmpty (sync_addin_id);
+		}
+
+		Gtk.Menu GetRightClickMenu ()
+		{
+			if (tray.TomboyTrayMenu != null)
+				tray.TomboyTrayMenu.Hide ();
+
+			if (context_menu != null) {
+				context_menu.Hide ();
+				return context_menu;
+			}
+
+			context_menu = new Gtk.Menu ();
 
 			Gtk.AccelGroup accel_group = new Gtk.AccelGroup ();
-			menu.AccelGroup = accel_group;
+			context_menu.AccelGroup = accel_group;
 
 			Gtk.ImageMenuItem item;
 
+			sync_menu_item = new Gtk.ImageMenuItem (Catalog.GetString ("S_ynchronize Notes"));
+			sync_menu_item.Image = new Gtk.Image (Gtk.Stock.Convert, Gtk.IconSize.Menu);
+			UpdateMenuItems();
+			Preferences.SettingChanged += Preferences_SettingChanged;
+			sync_menu_item.Activated += SyncNotes;
+			context_menu.Append (sync_menu_item);
+
+			context_menu.Append (new Gtk.SeparatorMenuItem ());
+
 			item = new Gtk.ImageMenuItem (Catalog.GetString ("_Preferences"));
 			item.Image = new Gtk.Image (Gtk.Stock.Preferences, Gtk.IconSize.Menu);
 			item.Activated += ShowPreferences;
-			menu.Append (item);
+			context_menu.Append (item);
 
 			item = new Gtk.ImageMenuItem (Catalog.GetString ("_Help"));
 			item.Image = new Gtk.Image (Gtk.Stock.Help, Gtk.IconSize.Menu);
 			item.Activated += ShowHelpContents;
-			menu.Append (item);
+			context_menu.Append (item);
 
 			item = new Gtk.ImageMenuItem (Catalog.GetString ("_About Tomboy"));
 			item.Image = new Gtk.Image (Gtk.Stock.About, Gtk.IconSize.Menu);
 			item.Activated += ShowAbout;
-			menu.Append (item);
+			context_menu.Append (item);
 
-			menu.Append (new Gtk.SeparatorMenuItem ());
+			context_menu.Append (new Gtk.SeparatorMenuItem ());
 
 			item = new Gtk.ImageMenuItem (Catalog.GetString ("_Quit"));
 			item.Image = new Gtk.Image (Gtk.Stock.Quit, Gtk.IconSize.Menu);
 			item.Activated += Quit;
-			menu.Append (item);
+			context_menu.Append (item);
 
-			menu.ShowAll ();
-			return menu;
+			context_menu.ShowAll ();
+			return context_menu;
 		}
 
 		void ShowPreferences (object sender, EventArgs args)
@@ -259,6 +300,11 @@ namespace Tomboy
 			Tomboy.ActionManager ["ShowPreferencesAction"].Activate ();
 		}
 
+		void SyncNotes (object sender, EventArgs args)
+		{
+			Tomboy.ActionManager ["NoteSynchronizationAction"].Activate ();
+		}
+
 		void ShowHelpContents (object sender, EventArgs args)
 		{
 			Tomboy.ActionManager ["ShowHelpAction"].Activate ();
